'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.describeType = describeType;
exports.describeSubTypes = describeSubTypes;
exports.arrayOf = arrayOf;
exports.enumType = enumType;
exports.shape = shape;
exports.union = union;

var _markdownTable = require('markdown-table');

var _markdownTable2 = _interopRequireDefault(_markdownTable);

var _helpers = require('./helpers');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function describeType(type) {
  var level = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;

  switch (type.name) {
    case 'shape':
      return shape(type.value, level);
    case 'enum':
      return enumType(type.value);
    case 'union':
      return union(type.value, level);
    case 'arrayOf':
      return arrayOf(type.value, level);
    case 'custom':
      return type.raw || type.name;
    default:
      return type.value || type.name;
  }
}

function describeSubTypes(types) {
  var level = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;

  var keys = Object.keys(types);

  var subTypes = '';
  var index = 0;
  keys.forEach(function (key) {
    var prop = types[key];
    // Type can either be on the prop, or in the type field depending on depth.
    var type = (0, _helpers.getType)(prop) || prop;

    if ((0, _helpers.isComplexType)(type.name)) {
      var result = describeType(type, level);
      if (result) {
        // Generate a horizontal ruler
        if (level === 0 && index > 0) subTypes += '\n' + '-'.repeat(80) + '\n';

        // Output a header, with the correct level
        subTypes += '\n###' + '#'.repeat(Math.min(3, level)) + ' ' + key + '\nType: _' + (0, _helpers.getTypeName)(type) + '_\n';
        if (prop.description) subTypes += '\n' + prop.description + '\n';

        if (level === 0) {
          // Add description on first level
          var typeDescription = void 0;
          switch (type.name) {
            case 'arrayOf':
              typeDescription = '**' + key + '** is an array of the following type:';
              break;
            case 'shape':
              typeDescription = '**' + key + '** is an object with:';
              break;
            case 'enum':
              typeDescription = '**' + key + '** should be one of the following values:';
              break;
            case 'union':
              typeDescription = '**' + key + '** should be one of the following types:';
              break;
            default:
              break;
          }

          if (typeDescription) subTypes += '\n' + typeDescription + '\n';
        }

        subTypes += '' + result;
        index += 1;
      }
    }
  });

  return subTypes;
}

function arrayOf(type, level) {
  return describeType(type, level);
}

function enumType(values) {
  var items = [['Value', 'Type']].concat(_toConsumableArray(values.map(function (item) {
    var value = item.value;

    // Do some basic type detection, to better label the Enum. React-doc-gen doesn't provide the type of an enum, so guess
    var type = 'Object';
    if (value.charAt(0) === "'") {
      // Remove the ' in strings
      value = value.replace(/'/g, '');
      type = 'String';
    } else if (!isNaN(parseInt(value, 10))) type = 'Number';else if (value === 'true' || value === 'false') type = 'Boolean';else if (value === 'null' || value === 'undefined') type = value;
    return [value, type];
  })));

  return '\n' + (0, _markdownTable2.default)(items) + '\n';
}

function shape(types, level) {
  var output = '';
  var keys = Object.keys(types);

  var items = [['Name', 'Type', 'Required']].concat(_toConsumableArray(keys.map(function (key) {
    var type = types[key];
    return [(0, _helpers.getKey)(key, type), (0, _helpers.getTypeName)(type), type.required];
  })));

  output += '\n' + (0, _markdownTable2.default)(items) + '\n';

  var subTypes = describeSubTypes(types, level + 1);

  if (subTypes) {
    output += subTypes;
  }

  return output;
}

function union(types, level) {
  return types.reduce(function (output, type) {
    var result = '\n####' + '#'.repeat(Math.min(level, 2)) + ' ' + (0, _helpers.getTypeName)(type) + '\n';
    result += (0, _helpers.blockquote)(describeType(type, level + 1));
    result += '\n\n'; // Break the blockquote by adding double line break
    return output + result;
  }, '');
}